'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };
/**
 *  Fork Copyright (c) 2016, Martin Heidegger,
 *
 *  Original Copyright (c) 2015, Facebook, Inc.
 *  All rights reserved.
 *
 *  This source code is licensed under the BSD-style license found in the
 *  LICENSE file in the root directory of this source tree. An additional grant
 *  of patent rights can be found in the PATENTS file in the same directory.
 */

var _httpErrors = require('http-errors');

var _httpErrors2 = _interopRequireDefault(_httpErrors);

var _url = require('url');

var _url2 = _interopRequireDefault(_url);

var _crypto = require('crypto');

var _toBuffer = require('./toBuffer.js');

var _toBuffer2 = _interopRequireDefault(_toBuffer);

var _parse = require('./parse');

var _renderGraphiQL = require('./renderGraphiQL');

var _handler = require('./handler');

var _graphql = require('graphql');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Used to configure the graphqlHTTP middleware by providing a schema
 * and other configuration options.
 *
 * Options can be provided as an Object, a Promise for an Object, or a Function
 * that returns an Object or a Promise for an Object.
 */


/**
 * All information about a GraphQL request.
 */
function validateOptions(optionsData) {
  // Assert that optionsData is in fact an Object.
  if (!optionsData || (typeof optionsData === 'undefined' ? 'undefined' : _typeof(optionsData)) !== 'object') {
    throw new Error('GraphQL middleware option function must return an options object ' + 'or a promise which will be resolved to an options object.');
  }

  // Assert that schema is required.
  if (!optionsData.schema) {
    throw new Error('GraphQL middleware options must contain a schema.');
  }

  return optionsData;
}

/**
 * Middleware for express; takes an options object or function as input to
 * configure behavior, and returns an express middleware.
 */
module.exports = graphqlHTTP;
function graphqlHTTP(options) {
  var resolveOptions = createOptionResolver(options);

  return function (request, response) {
    // Higher scoped variables are used when finishing the request.

    // These options can be overwritten but need to be set to default
    // values in case resolving the options
    var pretty = false;
    var formatErrorFn = void 0;
    var showGraphiQL = false;

    // The input will be used to render showGraphiQl if given.
    var query = void 0;
    var variables = void 0;
    var operationName = void 0;

    // Promises are used as a mechanism for capturing any thrown errors during
    // the asynchronous process below.
    return resolveOptions(request, response).then(function (optionsData) {
      // GraphQL HTTP only supports GET and POST methods.
      if (request.method !== 'GET' && request.method !== 'POST') {
        response.setHeader('Allow', 'GET, POST');
        throw (0, _httpErrors2.default)(405, 'GraphQL only supports GET and POST requests.');
      }

      // Collect information from the options data object.
      pretty = optionsData.pretty;
      formatErrorFn = optionsData.formatError;

      // Parse the Request to get GraphQL request parameters.
      return getGraphQLParams(request, response, optionsData.getPrivateKey, optionsData.acceptedCipherAlgorithms || ['aes-256-ecb']).then(function (params) {
        // Get GraphQL params from the request and POST body data.
        query = params.query;
        variables = params.variables;
        operationName = params.operationName;
        showGraphiQL = optionsData.graphiql && (0, _handler.canDisplayGraphiQL)(request, params);

        // If there is no query, but GraphiQL will be displayed, do not produce
        // a result, otherwise return a 400: Bad Request.
        if (!query) {
          if (showGraphiQL) {
            return null;
          }
          throw (0, _httpErrors2.default)(400, 'Must provide query string.');
        }

        return (0, _parse.parseQuery)(optionsData.schema, query, optionsData.validationRules).then(function (documentAST) {

          // Only query operations are allowed on GET requests.
          if (request.method === 'GET') {
            // Determine if this GET request will perform a non-query.
            var operation = (0, _parse.getOperationType)(documentAST, operationName);
            if (operation !== 'query') {
              // If GraphiQL can be shown, do not perform this query, but
              // provide it to GraphiQL so that the requester may perform it
              // themselves if desired.
              if (showGraphiQL) {
                return null;
              }

              // Otherwise, report a 405: Method Not Allowed error.
              response.setHeader('Allow', 'POST');
              throw (0, _httpErrors2.default)(405, 'Can only perform a ' + operation + ' operation ' + 'from a POST request.');
            }
          }
          return (0, _handler.exec)(optionsData.schema, optionsData.rootValue, optionsData.context || request, optionsData.extensions, documentAST, variables, operationName);
        });
      });
    }).catch(function (error) {
      return (0, _handler.handleError)(response, error);
    }).then(function (result) {
      return (0, _handler.handleResult)(formatErrorFn, response, result);
    }).then(function (result) {
      // If allowed to show GraphiQL, present it instead of JSON.
      var eGraphQL = request.eGraphQL;
      if (eGraphQL !== null && eGraphQL !== undefined) {
        var data = {
          t: Date.now(),
          status: response.statusCode,
          payload: result
        };
        response.statusCode = 200;
        response.setHeader('Content-Type', 'application/egraphql');
        sendResponse(response, (0, _toBuffer2.default)(eGraphQL.cipher(data)));
      } else if (showGraphiQL) {
        var payload = (0, _renderGraphiQL.renderGraphiQL)({
          query: query, variables: variables,
          operationName: operationName, result: result
        });
        response.setHeader('Content-Type', 'text/html; charset=utf-8');
        sendResponse(response, payload);
      } else {
        // Otherwise, present JSON directly.
        var _payload = JSON.stringify(result, null, pretty ? 2 : 0);
        response.setHeader('Content-Type', 'application/json; charset=utf-8');
        sendResponse(response, _payload);
      }
    });
  };
}

module.exports.createOptionResolver = createOptionResolver;
function createOptionResolver(options) {
  if (!options) {
    throw new Error('GraphQL middleware requires options.');
  }
  if (typeof options === 'function') {
    var _ret = function () {
      var optionsFetch = options;
      return {
        v: function v(request, response) {
          return new Promise(function (resolve) {
            return resolve(optionsFetch(request, response));
          }).then(validateOptions);
        }
      };
    }();

    if ((typeof _ret === 'undefined' ? 'undefined' : _typeof(_ret)) === "object") return _ret.v;
  }
  validateOptions(options);
  return function () {
    return Promise.resolve(options);
  };
}

/**
 * Provided a "Request" provided by express or connect (typically a node style
 * HTTPClientRequest), Promise the GraphQL request parameters.
 */
module.exports.getGraphQLParams = getGraphQLParams;
function getGraphQLParams(request, response, getPrivateKey, acceptedCipherAlgorithms) {
  var cipherAlgorithm = String(request.headers['x-cipher'] || '');
  var keyID = String(request.headers['x-key-id'] || '');
  if (request.method === 'POST' && getPrivateKey && (keyID || cipherAlgorithm)) {
    return isEncryptedRequest(getPrivateKey, acceptedCipherAlgorithms, cipherAlgorithm, keyID).then(function (credentials) {
      return (0, _parse.parseBody)(request, 'utf-8', credentials.decipher).then(function (data) {
        request.eGraphQL = credentials;
        if (!data.payload) {
          return Promise.reject((0, _httpErrors2.default)(400, 'Payload missing'));
        }
        return (0, _parse.parseGraphQLParams)(data.payload);
      }).catch(function (e) {
        if (e.statusCode === 401) {
          return Promise.reject(e);
        }
        request.eGraphQL = credentials;
        return Promise.reject(e);
      });
    });
  }
  return getRegularGraphQLParams(request);
}

function isEncryptedRequest(getPrivateKey, acceptedCipherAlgorithms, cipherAlgorithm, keyID) {
  if (!keyID) {
    return Promise.reject((0, _httpErrors2.default)(400, 'The header "x-cipher" requires the use of "x-key-id".'));
  }
  if (!cipherAlgorithm) {
    return Promise.reject((0, _httpErrors2.default)(400, 'The header "x-key-id" requires the use of "x-cipher".'));
  }
  if (acceptedCipherAlgorithms.indexOf(cipherAlgorithm) === -1) {
    return Promise.reject((0, _httpErrors2.default)(400, '"x-cipher" set to "' + cipherAlgorithm + '" is not acceptable.\n' + ('Acceptable options are: ' + acceptedCipherAlgorithms.join(', ') + '.')));
  }
  return getPrivateKey(keyID).catch(function () {
    // should getPrivateKey fail for some reason its content could contain
    // potentially reveal information about how getPrivateKey works. In order
    // to cover that we just return with a delay.
    return delayedReject();
  }).then(function (privateKey) {
    if (!privateKey) {
      return delayedReject();
    }
    return {
      keyID: keyID,
      privateKey: privateKey,
      cipher: function cipher(data) {
        var c = (0, _crypto.createCipher)(cipherAlgorithm, privateKey);
        return Buffer.concat([c.update((0, _toBuffer2.default)(JSON.stringify(data))), c.final()]).toString('base64');
      },
      decipher: function decipher(data) {
        try {
          var d = (0, _crypto.createDecipher)(cipherAlgorithm, privateKey);
          var str = Buffer.concat([d.update((0, _toBuffer2.default)(data, 'base64')), d.final()]).toString();
          return Promise.resolve(JSON.parse(str));
        } catch (e) {
          // There is no simple way to figure out if the error occurs
          // because of malformatted JSON content or wrong encryption
          // so we need to go with authentication error since it is
          // not good to inform a possible hacker that the JSON might
          // be malformated (indicating that the user actually exists)
          // Delaying response
          return delayedReject();
        }
      }
    };
  });
}

function delayedReject() {
  // In order for a possible attacker to not know
  // if a request failed because the user exists
  // or not we need to reject with a random delay
  return new Promise(function (_, reject) {
    return setTimeout(function () {
      return reject((0, _httpErrors2.default)(401, 'Authentication failed.'));
    }, 50 + Math.random() * 500);
  });
}

function getRegularGraphQLParams(request) {
  return (0, _parse.parseRequest)(request).then(function (bodyData) {
    var urlData = request.url && _url2.default.parse(request.url, true).query || {};
    return (0, _parse.parseGraphQLParams)(Object.assign(urlData, bodyData));
  });
}

module.exports.eGraphqlCreateSecret = function (privateKey, secret) {
  return new Promise(function (resolve, reject) {
    return (0, _crypto.pbkdf2)(privateKey, (0, _toBuffer2.default)(secret), 100000, 256, 'sha512', function (err, newPrivateKey) {
      return err ? reject(err) : resolve(newPrivateKey);
    });
  });
};
module.exports.eGraphqlSessionMutation = function (createSession) {
  return {
    type: new _graphql.GraphQLObjectType({
      name: 'EncryptedGraphqlSession',
      fields: {
        keyID: {
          type: _graphql.GraphQLString
        },
        secret: {
          type: _graphql.GraphQLString
        }
      }
    }),
    resolve: function resolve(source, args, context) {
      return createSession(context.eGraphQL.keyID);
    }
  };
};

module.exports.parseGraphQLParams = _parse.parseGraphQLParams;

/**
 * Helper function for sending the response data. Use response.send it method
 * exists (express), otherwise use response.end (connect).
 */
function sendResponse(response, data) {
  if (typeof response.send === 'function') {
    response.send(data);
  } else {
    response.end(data);
  }
}