'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };
/**
 *  Fork Copyright (c) 2016, Martin Heidegger,
 *
 *  Original Copyright (c) 2015, Facebook, Inc.
 *  All rights reserved.
 *
 *  This source code is licensed under the BSD-style license found in the
 *  LICENSE file in the root directory of this source tree. An additional grant
 *  of patent rights can be found in the PATENTS file in the same directory.
 */

exports.parseRequest = parseRequest;
exports.parseQuery = parseQuery;
exports.parseBody = parseBody;
exports.parseGraphQLParams = parseGraphQLParams;
exports.getOperationType = getOperationType;

var _contentType = require('content-type');

var _contentType2 = _interopRequireDefault(_contentType);

var _httpErrors = require('http-errors');

var _httpErrors2 = _interopRequireDefault(_httpErrors);

var _readBody = require('./readBody');

var _readBody2 = _interopRequireDefault(_readBody);

var _parsers = require('./parsers');

var _parsers2 = _interopRequireDefault(_parsers);

var _graphql = require('graphql');

var _handler = require('./handler');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Provided a "Request" provided by express or connect (typically a node style
 * HTTPClientRequest), Promise the body data contained.
 */
function parseRequest(req) {

  // Skip requests without content types.
  if (req.headers['content-type'] === undefined) {
    return Promise.resolve({});
  }

  var typeInfo = _contentType2.default.parse(req);

  // Use the correct body parser based on Content-Type header.
  var parseFn = _parsers2.default[typeInfo.type];

  var body = req.body;

  // If express has already parsed a body as a string, and the content-type
  // was application/graphql, parse the string body.
  if (typeof body === 'string' && typeInfo.type === 'application/graphql') {
    return Promise.resolve(parseFn(body));
  }

  var charset = (typeInfo.parameters.charset || 'utf-8').toLowerCase();

  return parseBody(req, charset, parseFn);
}

/**
 * Parses a given query following a provided Schema definition. Can be extended
 * by passing in optional validationRules.
 */
function parseQuery(schema, query, validationRules) {
  // GraphQL source.
  var source = new _graphql.Source(query, 'GraphQL request');

  // Parse source to AST, reporting any syntax error.
  var documentAST = void 0;
  try {
    documentAST = (0, _graphql.parse)(source);
  } catch (syntaxError) {
    return Promise.reject((0, _handler.graphqlError)(400, [syntaxError]));
  }

  var allValidationRules = _graphql.specifiedRules;
  if (validationRules) {
    allValidationRules = allValidationRules.concat(validationRules);
  }

  // Validate AST, reporting any errors.
  var validationErrors = (0, _graphql.validate)(schema, documentAST, allValidationRules);

  if (validationErrors.length > 0) {
    return Promise.reject((0, _handler.graphqlError)(400, validationErrors));
  }

  return Promise.resolve(documentAST);
}

function parseBody(req, charset, parseFn) {
  var body = req.body;

  // If express has already parsed a body as a keyed object, use it.
  if ((typeof body === 'undefined' ? 'undefined' : _typeof(body)) === 'object' && !(body instanceof Buffer)) {
    return Promise.resolve(body);
  }

  // If no Content-Type header matches, parse nothing.
  if (!parseFn) {
    return Promise.resolve({});
  }

  // Already parsed body we didn't recognise? Parse nothing.
  if (body) {
    return Promise.resolve({});
  }

  return (0, _readBody2.default)(req, charset).then(parseFn);
}

/**
 * Helper function to get the GraphQL params from the request.
 */
function parseGraphQLParams(data) {
  // GraphQL Query string.
  var query = data.query;
  if (typeof query !== 'string') {
    query = null;
  }

  // Parse the variables if needed.
  var variables = data.variables;
  if (typeof variables === 'string') {
    try {
      variables = JSON.parse(variables);
    } catch (error) {
      throw (0, _httpErrors2.default)(400, 'Variables are invalid JSON.');
    }
  } else if ((typeof variables === 'undefined' ? 'undefined' : _typeof(variables)) !== 'object') {
    variables = null;
  }

  // Name of GraphQL operation to execute.
  var operationName = data.operationName;
  if (typeof operationName !== 'string') {
    operationName = null;
  }

  var raw = data.raw !== undefined;

  return { query: query, variables: variables, operationName: operationName, raw: raw };
}

function getOperationType(documentAST, operationName) {
  var operationAST = (0, _graphql.getOperationAST)(documentAST, operationName);
  if (!operationAST) {
    // No operation is basically same as a 'query' operation for no content
    return 'query';
  }
  return operationAST.operation;
}